/*****************************************************************************
*  Copyright 2001 - 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/licenses/old-license/gpl-2.0.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/
#include <asm/io.h>
#include <asm/arch/brcm_rdb_i2c_mm_hs.h>


/* 
 * Some of the symbols have changed names for Capri - rather than edit the rdb, just
 * redefine the legacy symbols for backward compatibility 
 */
#ifdef CONFIG_CAPRI
#define I2C_START I2C_MM_HS_CS_CMD_CMD_START_RESTART 
#define I2C_RESET I2C_MM_HS_CS_EN_CMD_RESET_BSC
#define I2C_DIV1  I2C_MM_HS_TIM_PRESCALE_CMD_DIV1
#define I2C_STOP  I2C_MM_HS_CS_CMD_CMD_STOP
#define I2C_READ  I2C_MM_HS_CS_CMD_CMD_READ_BYTE 
#else
#define I2C_START I2C_MM_HS_CS_CMD_CMD_GEN_START
#define I2C_RESET I2C_MM_HS_CS_EN_CMD_RST_BSC
#define I2C_DIV1  I2C_MM_HS_TIM_PRESCALE_CMD_NODIV
#define I2C_STOP  I2C_MM_HS_CS_CMD_CMD_GEN_STOP
#define I2C_READ  I2C_MM_HS_CS_CMD_CMD_RD_A_BYTE 
#endif

#define BSC_WRITE_REG_FIELD(addr,mask,shift,data) (writel((((data << shift) & (mask)) | ( readl(addr) & ~mask ) ) , addr)) 
#define BSC_READ_REG_FIELD(addr,mask,shift)       ((readl(addr) & mask ) >> shift )
#define BSC_WRITE_REG(addr,data)                  (writel(data,addr))
#define BSC_READ_REG(addr)                        (readl(addr))


#define BSC_DEBUG_PRINT  0
#if BSC_DEBUG_PRINT
#define bsc_dprintf(e, f, s...)    { printf(f, ## s); }
#else
#define bsc_dprintf(e, f, s...)
#endif

/**
* Supported BSC bus speeds
*****************************************************************************/
typedef enum{
    BSC_SPD_32K = 0,      /*< 32KHZ */
    BSC_SPD_50K,          /*< 50KHZ */
    BSC_SPD_100K,         /*< 100KHZ */
    BSC_SPD_230K,         /*< 230KHZ */
    BSC_SPD_380K,         /*< 380KHZ */
    BSC_SPD_400K,         /*< 400KHZ */
    BSC_SPD_430K,         /*< 430KHZ */
    BSC_SPD_HS,           /*< HIGH SPEED */
    BSC_SPD_100K_FPGA,    /*< 100KHZ based on a 26MHz incoming clock */
    BSC_SPD_400K_FPGA,    /*< 400KHZ based on a 26MHz incoming clock */
    BSC_SPD_HS_FPGA,       /*< HIGH SPEED based on a 26MHz incoming clock */
    BSC_SPD_MAXIMUM       /*< 460K, assume auto sense turn off. */
} BSC_SPEED_t;

/**
* Supported BSC mode
*****************************************************************************/
typedef enum{
    BSC_MODE_MASTER = 0, /*< Master mode */
    BSC_MODE_SLAVE       /*< Slave mode */
} BSC_MODE_t;

/**
* Supported BSC interrupts
*****************************************************************************/

/**
* BSC Command
*****************************************************************************/
typedef enum{
    BSC_CMD_NOACTION = 0, /*< NOACTION */
    BSC_CMD_START,        /*< START command */
    BSC_CMD_RESTART,      /*< RESTART command */
    BSC_CMD_STOP,         /*< STOP command */
    BSC_CMD_READ_ACK,     /*< READ ACK command */
    BSC_CMD_READ_NAK,     /*< READ NAK command */
    BSC_CMD_HS_STOP       /*< High speed STOP command */
} BSC_CMD_t;


static inline void bsc_set_bus_speed(unsigned int baseAddr, BSC_SPEED_t speed);
static inline void kona_bsc_init(unsigned int baseAddr);
static inline void bsc_disable_intr(unsigned int baseAddr, unsigned int mask);
static inline void bsc_clear_intr_status(unsigned int baseAddr, unsigned int mask);
static inline void bsc_set_FIFO(unsigned int baseAddr, unsigned char enable);
static inline void bsc_set_autosense(unsigned int baseAddr, unsigned char on);
static inline unsigned char bsc_get_timeout(unsigned int baseAddr);
static inline void bsc_enable_intr(unsigned int baseAddr, unsigned int mask);
static inline void bsc_deinit(unsigned int baseAddr);
static inline void bsc_stop(unsigned int baseAddr);
static inline void bsc_set_sda(unsigned int baseAddr, unsigned char state);
static inline void bsc_set_scl(unsigned int baseAddr, unsigned char state);
static inline void bsc_set_timeout(unsigned int baseAddr, unsigned char on);
static inline void kona_bsc_send_cmd(unsigned int baseAddr, BSC_CMD_t cmd);
static inline void bsc_start(unsigned int baseAddr);
static inline void bsc_set_tim(unsigned int baseAddr, unsigned int val);
static inline void bsc_start_highspeed(unsigned int baseAddr);
static inline void bsc_stop_highspeed(unsigned int baseAddr);
static inline unsigned int bsc_get_enabled_intr(unsigned int baseAddr);
static inline unsigned int bsc_read_intr_status(unsigned int baseAddr);
static inline unsigned int bsc_read_data(unsigned int baseAddr, unsigned char *pBuffer, unsigned int size);
static inline unsigned int bsc_write_data(unsigned int baseAddr, unsigned char *pBuffer, unsigned int size);
static inline unsigned char bsc_get_autosense(unsigned int baseAddr);
static inline unsigned char bsc_get_bus_status(unsigned int baseAddr);
static inline unsigned char bsc_get_ack(unsigned int baseAddr);
static inline unsigned char bsc_get_sda(unsigned int baseAddr);
static inline unsigned char bsc_get_scl(unsigned int baseAddr);
static inline void bsc_set_soft_reset(unsigned int baseAddr, unsigned char enable );
static inline unsigned char bsc_get_soft_reset_ready(unsigned int baseAddr);
static inline unsigned char bsc_get_soft_reset(unsigned int baseAddr);
static inline BSC_MODE_t bsc_set_mode(unsigned int baseAddr, BSC_MODE_t mode);

enum bsc_bus_speed
{
   BSC_BUS_SPEED_32K,
   BSC_BUS_SPEED_50K,
   BSC_BUS_SPEED_230K,
   BSC_BUS_SPEED_380K,
   BSC_BUS_SPEED_400K,
   BSC_BUS_SPEED_430K,
   BSC_BUS_SPEED_HS, /* high-speed */

   /* for FPGA where only the 26 MHz core clock is available */
   BSC_BUS_SPEED_100K_FPGA,
   BSC_BUS_SPEED_400K_FPGA,
   BSC_BUS_SPEED_HS_FPGA,

   /* used as internal array index so do not reorder this */
   BSC_BUS_SPEED_MAX,
};

/*
 * Board dependent configuration for the Broadcom BSC (I2C) adapter
 */
struct bsc_adap_cfg
{
   /* I2C bus speed in KHz */
   enum bsc_bus_speed speed;
};

/**
*
*  @brief  Initialize BSC for the passed BSC instance
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return
*****************************************************************************/
static inline void kona_bsc_init(unsigned int baseAddr)
{
   bsc_dprintf( CDBG_INFO, "kona_bsc_init, base=0x%x\n", baseAddr );

   bsc_set_sda(baseAddr, 1);
   bsc_set_scl(baseAddr, 1);

   /* Time out */
   bsc_set_timeout( baseAddr, 1 );

   /* set to max 1023 cycles */
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET), I2C_MM_HS_TOUT_TOUT_LOW_MASK, I2C_MM_HS_TOUT_TOUT_LOW_SHIFT , 0x7F);
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET), I2C_MM_HS_TOUT_TOUT_HIGH_MASK, I2C_MM_HS_TOUT_TOUT_HIGH_SHIFT,0x3);

   /* FIFO */
   bsc_set_FIFO(baseAddr, 0);

   /* CRC */
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_RCM_OFFSET),I2C_MM_HS_RCM_EN_MASK , I2C_MM_HS_RCM_EN_SHIFT, 0);
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_RCM_OFFSET),I2C_MM_HS_RCM_LSB0_MASK, I2C_MM_HS_RCM_LSB0_SHIFT, 1);
   BSC_WRITE_REG((baseAddr+I2C_MM_HS_RCP_OFFSET), 0x89);

   /* Clear Interrupts */
   bsc_clear_intr_status(baseAddr, 0xff);

   /* Clear CMD */
   kona_bsc_send_cmd(baseAddr, BSC_CMD_NOACTION);

#ifdef BSC_HS
   /*pad control, slew rate 60ns<Tr(Tf)<100ns
     The PAD enable polarity is reverted. Need to use 0 instead of 1.*/
	BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_PADCTL_OFFSET), I2C_MM_HS_PADCTL_PAD_OUT_EN_MASK, I2C_MM_HS_PADCTL_PAD_OUT_EN_SHIFT , 0);
   /* start with non-HS, use 13MHz */
	BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TIM_OFFSET), I2C_MM_HS_TIM_PRESCALE_MASK , I2C_MM_HS_TIM_PRESCALE_SHIFT, I2C_DIV1);
	BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TIM_OFFSET), I2C_MM_HS_TIM_NO_DIV_MASK, I2C_MM_HS_TIM_NO_DIV_SHIFT, 0);
#endif

   /* start */
   bsc_start(baseAddr);
}

/**
*
*  @brief  De-Initialize BSC for the passed BSC instance
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline void bsc_deinit(unsigned int baseAddr)
{
   bsc_stop(baseAddr);
}

/**
*
*  @brief  Start BSC device
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/

static inline void bsc_start(unsigned int baseAddr)
{
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET), I2C_MM_HS_CLKEN_CLKEN_MASK, I2C_MM_HS_CLKEN_CLKEN_SHIFT, 1);
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_CMD_MASK, I2C_MM_HS_CS_CMD_SHIFT, I2C_MM_HS_CS_CMD_CMD_NO_ACTION);
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_ACK_MASK, I2C_MM_HS_CS_ACK_SHIFT, I2C_MM_HS_CS_ACK_CMD_GEN_START);
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_EN_MASK, I2C_MM_HS_CS_EN_SHIFT, I2C_MM_HS_CS_EN_CMD_ENABLE_BSC);
}

/**
*
*  @brief  Stop BSC device
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline void bsc_stop(unsigned int baseAddr)
{
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_EN_MASK, I2C_MM_HS_CS_EN_SHIFT, I2C_RESET);
   BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET), I2C_MM_HS_CLKEN_CLKEN_MASK, I2C_MM_HS_CLKEN_CLKEN_SHIFT, 0);
}

/**
*
*  @brief  Set BSC bus speed
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  speed   (in) speed to be set
*
*  @return none
*****************************************************************************/
/* Master clock is 13MHz */
#define BSCTIM_DIV_812500HZ     (0 << I2C_MM_HS_TIM_DIV_SHIFT)
#define BSCTIM_DIV_1625000HZ    (1 << I2C_MM_HS_TIM_DIV_SHIFT)
#define BSCTIM_DIV_3250000HZ    (2 << I2C_MM_HS_TIM_DIV_SHIFT)
#define BSCTIM_DIV_6500000HZ    (3 << I2C_MM_HS_TIM_DIV_SHIFT)

#ifndef FPGA 
#define BSC_HS_HSMODE_TIMING              0x00000513
#else
#define BSC_HS_HSMODE_TIMING              0x00000001
#endif

static inline void bsc_set_bus_speed(unsigned int baseAddr, BSC_SPEED_t speed)
{
    unsigned char DIV = 0, M = 0, N = 0, P = 0,NO_DIV=0,PRESCALE=I2C_DIV1;
    NO_DIV=0;
    PRESCALE=I2C_DIV1;
    bsc_dprintf( CDBG_INFO, "bsc_set_bus_speed, %d\n", speed);
    switch (speed)
    {
        case BSC_SPD_32K:
            M = 0x01;
            N = 0x01;
            DIV = BSCTIM_DIV_812500HZ;
            P = 0x04;
            break;
        case BSC_SPD_50K:
            M = 0x01;
            N = 0x01;
            DIV = BSCTIM_DIV_812500HZ;
            P = 0x02;
            break;
        case BSC_SPD_230K:
            M = 0x07;
            N = 0x00;
            DIV = BSCTIM_DIV_3250000HZ;
            P = 0x00;
            break;
        case BSC_SPD_380K:
            M = 0x02;
            N = 0x04;
            DIV = BSCTIM_DIV_6500000HZ;
            P = 0x01;
            break;
        case BSC_SPD_400K:
            M = 0x01;/* 2;*/
            N = 0x01;/* 2;*/
            NO_DIV=1;
            DIV = BSCTIM_DIV_3250000HZ;/*BSCTIM_DIV_6500000HZ; */
            P = 0x06;/* 0x01; */
            break;
        case BSC_SPD_430K:
            M = 0x03;
            N = 0x01;
            DIV = BSCTIM_DIV_6500000HZ;
            P = 0x01;
            break;
        case BSC_SPD_MAXIMUM:
            M = 0x02;
            N = 0x01;
            DIV = BSCTIM_DIV_6500000HZ;
            P = 0x01;
            break;
        case BSC_SPD_HS:
            M = 0x04;/*2;*/
            N = 0x01;/*2;*/
            DIV = BSCTIM_DIV_6500000HZ;
            PRESCALE=I2C_MM_HS_TIM_PRESCALE_CMD_DIV4;
            P = 0x05;/*0x01;*/
            BSC_WRITE_REG((baseAddr+I2C_MM_HS_HSTIM_OFFSET),BSC_HS_HSMODE_TIMING);
            break;
        /* master clock is 26MHz for FPGA */
        case BSC_SPD_100K_FPGA:
            M = 0x01;
            N = 0x01;
            DIV = 2;
            P = 0x06;
            PRESCALE = 2;
            break;
        case BSC_SPD_400K_FPGA:
            M = 0x04;
            N = 0x01;
            DIV = 0x3;
            P = 0x05;
            PRESCALE = 3;
            break;
        case BSC_SPD_HS_FPGA:
            M = 0x04;
            N = 0x01;
            NO_DIV = 0;
            DIV = 3;
            PRESCALE=I2C_DIV1;
            P = 0x07;
            BSC_WRITE_REG((baseAddr+I2C_MM_HS_HSTIM_OFFSET),BSC_HS_HSMODE_TIMING);
            break;
        case BSC_SPD_100K:
        default:
            M = 0x01;
            N = 0x01;
            DIV = BSCTIM_DIV_3250000HZ;/*BSCTIM_DIV_1625000HZ;*/
            P = 0x06; /*2;*/
            break;
    }

    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TIM_OFFSET), I2C_MM_HS_TIM_P_MASK, I2C_MM_HS_TIM_P_SHIFT, P);
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TIM_OFFSET), I2C_MM_HS_TIM_DIV_MASK, I2C_MM_HS_TIM_DIV_SHIFT, DIV);
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TIM_OFFSET), I2C_MM_HS_TIM_NO_DIV_MASK, I2C_MM_HS_TIM_NO_DIV_SHIFT, NO_DIV);
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TIM_OFFSET), I2C_MM_HS_TIM_PRESCALE_MASK, I2C_MM_HS_TIM_PRESCALE_SHIFT, PRESCALE);
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET), I2C_MM_HS_CLKEN_M_MASK, I2C_MM_HS_CLKEN_M_SHIFT, M);
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET), I2C_MM_HS_CLKEN_N_MASK, I2C_MM_HS_CLKEN_N_SHIFT, N);
}

/**
*
*  @brief  Get the current value of the BSC_TIM register (used for high-speed)
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return current value of the BSC_TIM register
*****************************************************************************/
static inline unsigned int bsc_get_tim(unsigned int baseAddr)
{
   return ( BSC_READ_REG(baseAddr+I2C_MM_HS_TIM_OFFSET) );
}

/**
*
*  @brief  Set the BSC_TIM register (used for high-speed)
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  baseAddr  (in) value to set
*
*  @return none
*****************************************************************************/
static inline void bsc_set_tim(unsigned int baseAddr, unsigned int val)
{
   BSC_WRITE_REG((baseAddr+I2C_MM_HS_TIM_OFFSET), val);
}

/**
*
*  @brief  Start High speed
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  speed     (in) speed to be set
*
*  @return none
*****************************************************************************/

static inline void bsc_start_highspeed(unsigned int baseAddr)
{
    unsigned int val;
#ifndef FPGA 
    val = (7 << I2C_MM_HS_TIM_P_SHIFT) |
          (I2C_DIV1 << I2C_MM_HS_TIM_PRESCALE_SHIFT) |
          (I2C_MM_HS_TIM_NO_DIV_MASK); /* set to 0x3c;*/
#else
    val = (7 << I2C_MM_HS_TIM_P_SHIFT) |
          (I2C_DIV1 << I2C_MM_HS_TIM_PRESCALE_SHIFT) |
          (I2C_MM_HS_TIM_DIV_CMD_DIV2);
#endif
    BSC_WRITE_REG((baseAddr+I2C_MM_HS_TIM_OFFSET),val);

    /* Configure Hs-mode timing register */
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_HSTIM_OFFSET), I2C_MM_HS_HSTIM_HS_MODE_MASK, I2C_MM_HS_HSTIM_HS_MODE_SHIFT, 1);
}

/**
*
*  @brief  Stop High speed
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  speed     (in) speed to be set
*
*  @return none
*****************************************************************************/
static inline void bsc_stop_highspeed(unsigned int baseAddr)
{
  /* Disable Hs-mode - leave other Hs-mode timing values untouched */
    BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_HSTIM_OFFSET), I2C_MM_HS_HSTIM_HS_MODE_MASK, I2C_MM_HS_HSTIM_HS_MODE_SHIFT, 0);
}


/**
*
*  @brief  Enable BSC FIFO
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  enable    (in) enable=1 to enable FIFO, enable=0 to
*                         disable FIFO
*
*  @return set mode
*****************************************************************************/
static inline void bsc_set_FIFO(unsigned int baseAddr, unsigned char enable)
{
    if (enable){
        BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TXFCR_OFFSET), I2C_MM_HS_TXFCR_FIFO_FLUSH_MASK, I2C_MM_HS_TXFCR_FIFO_FLUSH_SHIFT ,1);
        BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TXFCR_OFFSET), I2C_MM_HS_TXFCR_FIFO_EN_MASK ,I2C_MM_HS_TXFCR_FIFO_EN_SHIFT,1);
    }else{
        BSC_WRITE_REG((baseAddr+I2C_MM_HS_TXFCR_OFFSET),0);
    }
}

/**
*
*  @brief  Set BSC mode
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  mode      (in) mode to be set
*
*  @return set mode
*****************************************************************************/
static inline BSC_MODE_t bsc_set_mode(unsigned int baseAddr, BSC_MODE_t mode)
{
   (void) baseAddr;

    if (mode != BSC_MODE_MASTER)
    {
        bsc_dprintf(CDBG_ERRO, "ERROR: bsc_set_mode: no slave support\n");
    }

    return BSC_MODE_MASTER;
}

/**
*
*  @brief  Enable BSC interrupt
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  mask      (in) interrupts to enable
*
*  @return none
*****************************************************************************/
static inline void bsc_enable_intr(unsigned int baseAddr, unsigned int mask)
{
   /* clear pending interrupts */
   BSC_WRITE_REG((baseAddr+I2C_MM_HS_ISR_OFFSET), mask);

   BSC_WRITE_REG((baseAddr+I2C_MM_HS_IER_OFFSET), (bsc_get_enabled_intr(baseAddr) | mask) ) ;
}

/**
*
*  @brief  Disable BSC interrupt
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  mask      (in) interrupts to disable
*
*  @return none
*****************************************************************************/
static inline void bsc_disable_intr(unsigned int baseAddr, unsigned int mask)
{
	BSC_WRITE_REG((baseAddr+I2C_MM_HS_IER_OFFSET), (bsc_get_enabled_intr(baseAddr) & (~mask)));
}


/**
*
*  @brief  Get enabled BSC interrupt
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  mask      (in) interrupts to disable
*
*  @return none
*****************************************************************************/
static inline unsigned int bsc_get_enabled_intr(unsigned int baseAddr)
{
   return ( BSC_READ_REG(baseAddr+I2C_MM_HS_IER_OFFSET) & (~I2C_MM_HS_IER_RESERVED_MASK) );
}


/**
*
*  @brief  Read BSC interrupt status
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return status mask
*
*  @note
*****************************************************************************/
static inline unsigned int bsc_read_intr_status(unsigned int baseAddr)
{
   return( BSC_READ_REG(baseAddr+I2C_MM_HS_ISR_OFFSET) & (~I2C_MM_HS_ISR_RESERVED_MASK) );
}


/**
*
*  @brief  Clear BSC interrupt status
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return status mask
*
*  @note
*****************************************************************************/
static inline void bsc_clear_intr_status(unsigned int baseAddr, unsigned int mask )
{
   BSC_WRITE_REG( (baseAddr+I2C_MM_HS_ISR_OFFSET), (bsc_read_intr_status(baseAddr) & (mask)) );
}

/**
*
*  @brief  Send a cmd to BSC controller
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  cmd       (in) cmd to be executed
*
*  @return none
*****************************************************************************/
static inline void kona_bsc_send_cmd(unsigned int baseAddr, BSC_CMD_t cmd)
{
	unsigned char temp;

	temp = BSC_READ_REG((baseAddr+I2C_MM_HS_CS_OFFSET));

    bsc_dprintf( CDBG_INFO, "bsc_send_cmd, %d\n", cmd);
    switch (cmd) {
        case BSC_CMD_NOACTION:
            temp = (temp & ~I2C_MM_HS_CS_CMD_MASK) |
                (I2C_MM_HS_CS_CMD_CMD_NO_ACTION << I2C_MM_HS_CS_CMD_SHIFT);
			break;
		case BSC_CMD_START:
            temp = (temp & ~I2C_MM_HS_CS_ACK_MASK & ~I2C_MM_HS_CS_CMD_MASK) |
                (I2C_START << I2C_MM_HS_CS_CMD_SHIFT);
			break;
		case BSC_CMD_RESTART:
            temp = (temp & ~I2C_MM_HS_CS_CMD_MASK) | I2C_MM_HS_CS_ACK_MASK |
                (I2C_START << I2C_MM_HS_CS_CMD_SHIFT);
			break;
        case BSC_CMD_STOP:
            temp = (temp & ~I2C_MM_HS_CS_CMD_MASK) |
                I2C_STOP << I2C_MM_HS_CS_CMD_SHIFT;
			break;

        case BSC_CMD_READ_ACK:
            temp = (temp & ~I2C_MM_HS_CS_ACK_MASK & ~I2C_MM_HS_CS_CMD_MASK) |
                (I2C_READ << I2C_MM_HS_CS_CMD_SHIFT);
            break;
        case BSC_CMD_READ_NAK:
            temp = (temp & ~I2C_MM_HS_CS_CMD_MASK) | I2C_MM_HS_CS_ACK_MASK |
                (I2C_READ << I2C_MM_HS_CS_CMD_SHIFT);
            break;
        case BSC_CMD_HS_STOP:
            /* For high speed stop, need to set different value for BSC_TIM */
            {
                unsigned int val;
                /* Configure PRESCALE, p & DIV bit field */
                val = (I2C_MM_HS_TIM_PRESCALE_CMD_DIV4 << I2C_MM_HS_TIM_PRESCALE_SHIFT) | (  5 << I2C_MM_HS_TIM_P_SHIFT ) \
                    | ( I2C_MM_HS_TIM_DIV_CMD_DIV2 );
                BSC_WRITE_REG((baseAddr+I2C_MM_HS_TIM_OFFSET),val);
            }
            temp = (temp & ~I2C_MM_HS_CS_CMD_MASK) |
                I2C_STOP << I2C_MM_HS_CS_CMD_SHIFT;
            break;
    }

	BSC_WRITE_REG((baseAddr+I2C_MM_HS_CS_OFFSET), temp);
}

/**
*
*  @brief  Read data from BSC FIFO
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  pBuffer   (i/o) Buffer for data read
*  @param  size      (in) Buffer size
*
*  @return Number of bytes read
*****************************************************************************/
static inline unsigned int bsc_read_data(unsigned int baseAddr, unsigned char *pBuffer, unsigned int size)
{
   (void) size;

	pBuffer[0] = BSC_READ_REG(baseAddr+I2C_MM_HS_DAT_OFFSET);

    /* no RX FIFO */
    return 1;
}

/**
*
*  @brief  Write data to BSC FIFO
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  pBuffer   (in) Buffer to write data from
*  @param  size      (in) Buffer size
*
*  @return Number of bytes written
*****************************************************************************/

static inline unsigned int bsc_write_data(unsigned int baseAddr, unsigned char *pBuffer, unsigned int size)
{
    uint16_t i=0;
    for(i=0;i<size;i++) {
        BSC_WRITE_REG((baseAddr+I2C_MM_HS_DAT_OFFSET), pBuffer[i]);
    }
    return size;
}

/**
*
*  @brief  Set timeout
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline void bsc_set_timeout(unsigned int baseAddr, unsigned char on)
{
   if (on)
	  BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET),  I2C_MM_HS_TOUT_TE_MASK,  I2C_MM_HS_TOUT_TE_SHIFT, 1 ) ;
   else
	  BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET),  I2C_MM_HS_TOUT_TE_MASK,  I2C_MM_HS_TOUT_TE_SHIFT, 0 ) ;
}

/**
*
*  @brief  Get timeout
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline unsigned char bsc_get_timeout(unsigned int baseAddr)
{
    unsigned char temp;

	temp = BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET), I2C_MM_HS_TOUT_TE_MASK, I2C_MM_HS_TOUT_TE_SHIFT);

    return temp;
}

/**
*
*  @brief  Get auto sense status
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return TRUE for on; FALSE for off
*****************************************************************************/
static inline unsigned char bsc_get_autosense(unsigned int baseAddr)
{
    unsigned char temp;

	temp = BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET), I2C_MM_HS_CLKEN_AUTOSENSE_OFF_MASK, I2C_MM_HS_CLKEN_AUTOSENSE_OFF_SHIFT );

    return (!(temp));
}

/**
*
*  @brief  Set auto sense status
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline void bsc_set_autosense(unsigned int baseAddr, unsigned char on)
{
    if (on)
    {
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET),I2C_MM_HS_CLKEN_AUTOSENSE_OFF_MASK , I2C_MM_HS_CLKEN_AUTOSENSE_OFF_SHIFT, 0);
      /* When autosense is ON, a device may stretch clock very long time
		   We use max timeout value (1023 6.5MHz cycles)*/
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET), I2C_MM_HS_TOUT_TOUT_LOW_MASK, I2C_MM_HS_TOUT_TOUT_LOW_SHIFT, 0x7F);
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_TOUT_OFFSET), I2C_MM_HS_TOUT_TOUT_HIGH_MASK,I2C_MM_HS_TOUT_TOUT_HIGH_SHIFT, 0x7);
		bsc_set_timeout(baseAddr, 1);
    }
    else
    {
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CLKEN_OFFSET), I2C_MM_HS_CLKEN_AUTOSENSE_OFF_MASK, I2C_MM_HS_CLKEN_AUTOSENSE_OFF_SHIFT, 1);
		/* Timeout feature is not used when autosense is off */
		bsc_set_timeout(baseAddr, 0);
    }
}

/**
*
*  @brief  Get bus status
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline unsigned char bsc_get_bus_status(unsigned int baseAddr)
{
	unsigned char temp;

	temp = BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_TXCOUNT_OFFSET), I2C_MM_HS_TXCOUNT_STATUS_MASK, I2C_MM_HS_TXCOUNT_STATUS_SHIFT);

	return temp;
}

/**
*
*  @brief  Get ACK after writing a byte
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return none
*****************************************************************************/
static inline unsigned char bsc_get_ack(unsigned int baseAddr)
{
	unsigned char temp;

	temp = BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_ACK_MASK, I2C_MM_HS_CS_ACK_SHIFT);

   /* ACK is active low */
   if (temp)
      return 0;
   else
      return 1;
}

/**
*
*  @brief  Get SDA line state for Bit-Banging implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return STA line state
*****************************************************************************/
static inline unsigned char bsc_get_sda(unsigned int baseAddr)
{
	unsigned char temp;

	temp = BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET),I2C_MM_HS_CS_SDA_MASK , I2C_MM_HS_CS_SDA_SHIFT);

    return temp;
}

/**
*
*  @brief  Set SDA line for Bit-Banging implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  state     (in) state to be set
*
*  @return none
*****************************************************************************/
static inline void bsc_set_sda(unsigned int baseAddr, unsigned char state)
{
    if (state)
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_SDA_MASK, I2C_MM_HS_CS_SDA_SHIFT, 1 ) ;
    else
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_SDA_MASK, I2C_MM_HS_CS_SDA_SHIFT, 0 ) ;
}

/**
*
*  @brief  Get SCL line state for Bit-Bangling implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*
*  @return SCL line state
*****************************************************************************/
static inline unsigned char bsc_get_scl(unsigned int baseAddr)
{
	unsigned char temp;

	temp = BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_SCL_MASK, I2C_MM_HS_CS_SCL_SHIFT );

    return temp;
}

/**
*
*  @brief  Set SCL line for Bit-Bangling implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  state     (in) state to be set
*
*  @return none
*****************************************************************************/
static inline void bsc_set_scl(unsigned int baseAddr, unsigned char state)
{
    if (state)
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_SCL_MASK, I2C_MM_HS_CS_SCL_SHIFT, 1 ) ;
	else
		BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_CS_OFFSET), I2C_MM_HS_CS_SCL_MASK, I2C_MM_HS_CS_SCL_SHIFT, 0 ) ;
}


/**
*
*  @brief  Set SCL line for Bit-Bangling implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  state     (in) state to be set
*
*  @return none
*****************************************************************************/
static inline void bsc_set_soft_reset(unsigned int baseAddr, unsigned char enable )
{
   if( enable )
      BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_SFTRST_OFFSET),I2C_MM_HS_SFTRST_SWRST_MASK , I2C_MM_HS_SFTRST_SWRST_SHIFT, 1);
   else
      BSC_WRITE_REG_FIELD((baseAddr+I2C_MM_HS_SFTRST_OFFSET),I2C_MM_HS_SFTRST_SWRST_MASK , I2C_MM_HS_SFTRST_SWRST_SHIFT, 0);
}

/**
*
*  @brief  Set SCL line for Bit-Bangling implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  state     (in) state to be set
*
*  @return none
*****************************************************************************/
static inline unsigned char bsc_get_soft_reset(unsigned int baseAddr)
{
   return( BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_SFTRST_OFFSET), I2C_MM_HS_SFTRST_SWRST_MASK, I2C_MM_HS_SFTRST_SWRST_SHIFT));
}

/**
*
*  @brief  Set SCL line for Bit-Bangling implementation
*
*  @param  baseAddr  (in) mapped address of this BSC instance
*  @param  state     (in) state to be set
*
*  @return none
*****************************************************************************/
static inline unsigned char bsc_get_soft_reset_ready(unsigned int baseAddr)
{
   return( BSC_READ_REG_FIELD((baseAddr+I2C_MM_HS_SFTRST_OFFSET), I2C_MM_HS_SFTRST_SWRST_RDY_MASK, I2C_MM_HS_SFTRST_SWRST_RDY_SHIFT));
}


